BILL WICKES' EXPLANATION of "Vectored Enter":

Author: [William C Wickes]
Subject: HP-48SX Vectored Enter

   The HP-48SX manuals do not document a very powerful feature that we call
"Vectored ENTER," that allows you in effect to redefine or bypass the
command line parser and to have a shot at the stack etc. after the command
line has been executed.

   Keys that execute an automatic ENTER perform a two-step process:

   1.  The command line is parsed and evaluated.

   2.  The key definition is executed.

   When flags -62 and -63 are both set, the system extends this process as
follows:

   1.  The current path is searched for a global variable named *aENTER
       (here "*a" is the Greek alpha character--character 140).  If
       present, the command line is entered as a string object and *aENTER
       is executed.  If absent, the command line is parsed and evaluated
       normally.

   2.  The key definition is executed.

   3.  The current path is searched for a global variable named *bENTER
       ("*b" is Greek beta--character 223). If present, then a string
       representing the key definition is put on the stack, and *bENTER is
       executed.  The string is the key definition object's name if it is a
       command, XLIB name, global or local name, or an empty string for
       other object types; its primary purpose is to implement things like
       the TRACE mode on other calcs, where you can print a running record
       of what you do.

............................................................................

Example:

If you type "45" and press "SIN", the string "45" is pushed on the stack and 
*aENTER is executed.  After saving a copy of the command string, *aENTER 
executes OBJ->, which parses and evaluates "45" (applied to a string, OBJ-> 
is really the command line parser--here, it pushes the number 45 on the 
stack).  The stack is now ready for the function key SIN. The 48SX then 
executes SIN, the key that got things going. Finally, it pushes the string 
"SIN" on the stack and executes *bENTER.
 
In it's simplest form a trace function will work by defining *aENTER and 
*bENTER as follows:
 
 
 '\GaENTER'                      @ STO as alpha-ENTER
 \<< PR1 OBJ\-> \>>              @ prints the command line
 
 
 '\GbENTER'                      @ STO as beta-ENTER
 \<< PR1 DROP \>>                @ prints the "action" key that was pressed
 
 'TRON'
 \<< -62 SF -63 SF \>>           @ turns on vectored enter (begins trace)
 
 'TROFF'
 \<< -62 CF -63 CF \>>           @ turns off vectored enter (ends trace)
 

This is as simple as it gets, and it works fine. It will not print the result 
however.  To print the result all that is needed is to modify *b-ENTER to \<< 
PR1 DROP PR1 \>>, where the last PR1 prints the level 1 result.
 
Now, one can get a little fancier by including a "***" string next to each 
level 1 result print. Next, it would be nice to "protect" the *aENTER and 
*bENTER routines for the trace function from being "over written" by a 
different set of alpha & beta ENTER routines that a user may want to create 
for a different purpose.  The following program code takes care of all of 
this by setting up menu keys for the custom (CST) menu that create *aENTER 
and *bENTER whenever TRON is activated from the custom menu. No need to worry 
that it's been wiped out by something else.
 

The following are custom menu key definitions for trace-on (TRON) and
trace-off (TROFF). Page 215 of the HP48 User Manual explains how to use
them.
 
{ "TRON"                               @ menu key for TRON
\<<

  \<< PR1 OBJ\-> \>> '\GaENTER' STO    @ create and STO a-enter
  
  \<< PR1 DROP                         @ create and STO b-enter
      IFERR DUP                        @ include printing result with ***
        THEN                           @ DUP will error if no result,
        ELSE "  ***" + PR1 DROP        @ so just ignore it
      END 
  \>> 
      '\GbENTER' STO                   @ here we save b-ENTER
      -62 SF -63 SF                    @ turn on flags -62 and -63
      0 MENU                           @ return to last menu for convenience
\>> }
 
{ "TROFF"                              @ menu key for TROFF
\<< -62 CF -63 CF 0 MENU \>> }         @ turn off flags -62 and -63
                                       @ return to last menu for convenience
 
These two lists should be included in the LIST object (variable) that contains
your custom menu (CST by default, but could be something else that is stored
into CST to activate it).

Enjoy...

Lee Buck
Compuserve: 76430,1067 
