*****************************************************************************
* Modulname:	TETRIS
* Modultype:	Library
* Dest.Comp.:	HP48
* Language:	System RPL
* Author:	Detlef Mueller, Bellerbek 33, 22559 Hamburg, Germany
* Interface:	TETRIS
* Description:	This library uses the number 769 because nobody would really
*		be interested in the CASINO48 module (IMHO).
* To do:
* Edition History :
*	0.000 08-08-1991 DM	alpha version
*	0.100 08-17-1991 DM	change to bint
*	1.000 08-24-1991 DM	1st offical release
*	  100 08-25-1991 DM	minor changes
*	  200 08-25-1991 DM	correct/condense stone data
*	  300 09.09-1991 DM	add pause screen
*	  400 09-19-1991 DM	fix some things
*	  401 12-22-1991 DM	fix ERRTRAP, first line clear
*	  402 01-21-1992 DM	Georg-version.Lines marked with ( ** )
*	1.500 01-24-1992 DM	minor changes, release
*	  501 12-18-1992 DM	add high scores, make splitable
*	  502 06-06-1993 DM	add GX fix, swap PAUSE/QUIT keys
* Copyright :
*	(c) 1991-1993 by Detlef Mueller. ALL rights reserved.
*****************************************************************************

    TITLE	TETRIS

*****************************************************************************
*		INCLUDE FILES
*****************************************************************************


    INCLUDE	keydefs.h
    INCLUDE	rpl_42.h

*****************************************************************************
*		ROM ID DEFINITIONS -> 769
*****************************************************************************


xROMID		301

ASSEMBLE
RomId		EQU	#301
RPL


*****************************************************************************
*		EXTERNAL DEFINITIONS
*****************************************************************************


EXTERNAL	xTETRIS

EXTERNAL	xScoreTET
EXTERNAL	GetHSTab
EXTERNAL	StoHS
EXTERNAL	DrawHSTab
EXTERNAL	$>dGROB

EXTERNAL	Install
EXTERNAL	Play
EXTERNAL	Byebye
EXTERNAL	UpdatVal
EXTERNAL	LinesUp
EXTERNAL	LevelUp
EXTERNAL	StIsDown
EXTERNAL	Actions
EXTERNAL	ChkKeys
EXTERNAL	MoveStX
EXTERNAL	Pause
EXTERNAL	StoneHandler
EXTERNAL	Stones


*****************************************************************************
*		DEFINES
*****************************************************************************


DEFINE	SetSt		1GETLAM EVAL	( *'Set stone'* )
DEFINE	TstSt		2GETEVAL	( *'Test stone'* )
DEFINE	@Timer		3GETLAM		( *Timing value* )
DEFINE	!Timer		3PUTLAM
DEFINE	@NxtStIm	4GETLAM		( *Next stone image* )
DEFINE	!NxtStIm	4PUTLAM
DEFINE	@NxtStNo	5GETLAM		( *Next stone number* )
DEFINE	!NxtStNo	5PUTLAM
DEFINE	@Level		6GETLAM		( *Level* )
DEFINE	!Level		6PUTLAM
DEFINE	@Score		7GETLAM		( *Score* )
DEFINE	!Score		7PUTLAM
DEFINE	@Lines		8GETLAM		( *Full lines* )
DEFINE	!Lines		8PUTLAM
DEFINE	@StNo		9GETLAM		( *Stone number* )
DEFINE	!StNo		9PUTLAM
DEFINE	@StDir		10GETLAM	( *Stone direction* )
DEFINE	!StDir		10PUTLAM
DEFINE	@StFt		11GETLAM	( *Stone falltime* )
DEFINE	!StFt		11PUTLAM
DEFINE	@A		12GETLAM	( *Temporary* )
DEFINE	!A		12PUTLAM

DEFINE	ClrDrpReq	ONE ClrUserFlag	( *Stone drop request flag* )
DEFINE	SetDrpReq	ONE SetUserFlag
DEFINE	DrpReq?		ONE TestUserFlag
DEFINE	ClrQuit		TWO ClrUserFlag	( *Quit game request* )
DEFINE	SetQuit		TWO SetUserFlag
DEFINE	Quit?		TWO TestUserFlag
DEFINE	ClrPreV		THREE ClrUserFlag ( *Preview on/off* )
DEFINE	SetPreV		THREE SetUserFlag
DEFINE	PreV?		THREE TestUserFlag


*****************************************************************************
*		OBJECTS
*****************************************************************************


*** TEcfg *******************************************************************
* Interface :	( --> )
* Description :	Attach this library
* Ed. History :
*     1.000	07.08.1991	DM	start
*****************************************************************************

ASSEMBLE
=TEcfg
RPL
::
    [#] RomId TOSRRP			( *Autoattach* )
;

*** xTETRIS *****************************************************************
* Interface :	( --> )
* Description :	Main program TETRIS
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

ASSEMBLE
	CON(1)	8		* Tell parser 'Non algebraic'
RPL
xNAME	TETRIS
::
    CK0
    ClrDA1IsStat RECLAIMDISP TURNMENUOFF
    ERRSET
    ::	Install			( *Install TETRIS* )
	Play			( *Do it* )
	Byebye			( *Game over ..* )
    ;
    ERRTRAP
    ::	TURNMENUON RECLAIMDISP	( *On error enshure correct display size* )
	ERRJMP
    ;
;

*** xScoreTET ***************************************************************
* Interface :	( --> )
* Description :	Create/display highscorelist
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

DEFINE  HSName          ID TETpar

ASSEMBLE
	CON(1)  8
RPL
xNAME   ScoreTET		( --> )
::
    CK0
    GetHSTab			( --> {} )
    DrawHSTab
;

*** DrawHSTab ***************************************************************
* Interface :	( {} --> )
* Description :	Display the high score table
* Ed. History :
*     1.000	18.12.1992	DM	start
*****************************************************************************

NULLNAME    DrawHSTab
::
    INCOMPDROP			( {hs}' --> {1} .. {6} )

    RECLAIMDISP
    ELEVEN ZERO "TETRIS High Scores"

**	NULLNAME        $>dGROB
**	::
    TRUE SWAP $>GROB DUPONE ZERO GROB+#
    UNROT HARDBUFF UNROT GROB!
**	;

    SIX
    #1+_ONE_DO (DO)
	SEVEN INDEX@#- ROLL INCOMPDROP
	#>$ OVERLEN$ OVERLEN$ #+ TWENTYTWO SWAP#- Blank$
	SWAP&$ &$ INDEX@ #1+ DISPN
    LOOP

    SetDA12Temp
;

*** GetHSTab ****************************************************************
* Interface :	( --> {} )
* Description :	Create/get the high score table
* Ed. History :
*     1.000	18.12.1992	DM	start
*****************************************************************************

NULLNAME	GetHSTab
::
    ' HSName SAFE@_HERE
    ITE
    ::  CK&DISPATCH0
	# AF
	::				( ld --> {} )
	    TOTEMPOB
	    [#] RomId SWAP		( --> #rid ld )

    CODE
*			A	C	D1
	A=DAT1	A	->ld		->stk
	AD1EX		->stk		->ld
	LC(5)	=DOLIST		{}prlg
	DAT1=C	A	** LDprlg becomes {}prlg
	D1=D1+	5			->len
	LC(5)	=DOBINT		#prlg
	DAT1=C	A	** len becomes #prlg
	D1=A				->stka
	GOVLNG	=Loop
    ENDCODE

	    DUP CDRCOMP			( #rid {} --> #rid {} {}-1st )
	    SWAP CARCOMP		( --> #rid {}-1st #rid' )

	    ROT #=
	;
	ZERO
	    FALSE
    ;
	ONEFALSE

    ?SEMI

    DROP NULL$ ZERO TWO{}N SIX NDUPN {}N

    StoHS
;

*** StoHS *******************************************************************
* Interface :	( --> {} )
* Description :	Store high score table into current directory
* Ed. History :
*     1.000	18.12.1992	DM	start
*****************************************************************************

NULLNAME        StoHS	( {} --> {} )
::
    DUP
    [#] RomId >HCOMP		( --> { #rid ... } )
    DUP OSIZE #5-		( --> {} #sz )

    CODE			( {} #sz --> LibDat )
* CPU			A	C	D1
	GOSBVL	=POP#	sz		->stk
	C=A	A		sz
	RSTK=C		** Save size
	A=DAT1	A	->{}
	AD1EX		->stk		->{}
	LC(5)	=DOEXT0		LD
	DAT1=C	A	** {}prlg becomes LDprlg
	D1=D1+	5			->#prlg
	C=RSTK			sz
	DAT1=C	A	** #prlg becomes length
	D1=A				->stk
	GOVLNG	=Loop
    ENDCODE

    ' HSName ?STO_HERE
;

*** Install *****************************************************************
* Interface :	( --> )
* Description :	Setup TETRIS environment, init game screen
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME	Install
::
    ZEROZEROZERO ZEROZEROZERO ZERO	( *Initial: A, falltime, stone dir,* )
					( *nr, lines, score, level* )
    %RAN %7 %* %IP># #1+DUP		( *Determine first stone nr.* )
    Stones SWAP NTHCOMPDROP CARCOMP	( *Determine first stone image* )
    % 0.19				( *Timer value = .19 - lev * .0171* )
    StoneHandler INCOMPDROP		( *Tst/SetStone* )
    ' NULLLAM TWELVE NDUPN {}N BIND	( *Generate temporary environment* )
					( *Setup screen* )
    FORTYTHREE ZEROOVER SIXTY		( *Draw field* )
    84 ZEROOVER 4PICK
    6PICK OVER 4PICK OVER
    LINEON LINEON LINEON
    110 FIFTYSIX "PAUSE" MakeStdLabel	( *Menue lables* )
    88 FIFTYSIX "+LEVL" MakeStdLabel
    TWENTYTWO FIFTYSIX "PVT" MakeStdLabel
    ZERO FIFTYSIX "QUIT" MakeStdLabel
    TWO THIRTYSIX "LINES" $>grob	( *Score field* )
    TWO NINETEEN "LEVEL" $>grob
    TWO DUP "SCORE" $>grob
    91 TWO				( *Help screen* )
    ASSEMBLE
	MKGROB	38,48
	NIBHEX	FFFFFFFFF35240408092FFFFFFFFF30000000000F7FBFBF4E180104805128010
	NIBHEX	480510801048051080F148F4E180104824028010484402801048841280F34805
	NIBHEX	E10000000000FFFFFFFFF35240408092FFFFFFFFF300000000000002E1000000
	NIBHEX	06120000000E040000000E140000000E34000000000000008CFFFFF720C40404
	NIBHEX	0460E4A44494E0F5A4E4B4F1E4A4F5F4E0C44CF7D46084440494200404040400
	NIBHEX	0CFFFFF700048404240004C404640004ECF7E40004F4F5E50004E4E4E40004C4
	NIBHEX	44640004840424000CFFFFF7000000000000000040000000004000000008F300
	NIBHEX	000000F100000000E000000000400000
    RPL

    EIGHT
    ZERO_DO (DO)
	XYGROBDISP
    LOOP

    FORTYTWO FIFTYSEVEN OVER SIXTYTHREE
    LINEOFF				( *Clr. last col. of 'PVT'* )
    NINE TWENTYSIX FORTYTHREE		( *Display 0 score/level/lines* )

    THREE
    ZERO_DO (DO)
	ZERO UpdatVal
    LOOP

    ClrQuit SetPreV			( *Init. flags* )
    Pause				( *Initial screen* )
;

*** Play ********************************************************************
* Interface :	( --> )
* Description :	Inner TETRIS loop
* Modifies :	StNo StDir NxtStNo NxtStIm StFt Quit
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME	Play
::
    BEGIN
	@NxtStIm @NxtStNo !StNo
	ONE !StDir

	Stones %RAN %7 %* %IP># #1+DUP	( *Determine, ..* )
	!NxtStNo NTHCOMPDROP CARCOMP	( *.. store next stone nr, get st. blocks* )
	!NxtStIm			( *Store next stone image* )

	PreV? IT
	::  #C06 ZERO SetSt		( *Clr preview field* )
	    @NxtStIm SWAP
	    #F99F SetSt 2DROP		( *Preview stone at [6,12]* )
	;

	ZERO !StFt			( *Reset falltime* )
	ClrDrpReq			( *Clr drop flag* )
	FOURTEEN			( *Start stone at [14,0]* )

	2DUP TstSt #0=ITE		( *Stone has room ?* )
	::				( *Yes* )
	    #F99F SetSt			( *Draw stone in 1st position* )

	    BEGIN			( --> hxs stim hxs p )
		DrpReq? ?SKIP		( *Skip key's if drop req.* )
		::  THREE
		    ZERO_DO (DO)
			ChkKeys		( *Check for pending keys* )
		    LOOP
		    @StFt #1+ !StFt	( *Update falltime* )
		    ATTN? NOT?SEMI
		    SetQuit		( *Attn, quick out* )
		;
		Quit? ?SKIP		( *Skip on quit req.* )
		::  ZERO SetSt		( *Undraw stone* )
		    REALOBOB #+		( *y += 1* )
		    2DUP TstSt
		    #0=case		( *Stone down ?* )
		    ::	#F99F SetSt	( *No, draw stone* )
		    ;
					( *Stone is down* )
		    REALOBOB #- #F99F
		    SetSt		( *Draw st. in last pos* )
		    DrpReq? case	( *Drop requested ?* )
		    ::  ClrDrpReq	( *Yes, chance to move stone in x* )
		    ;

		    2DROP ZERO		( hxs[si] #[pos] --> # *Signal down* )
		;
		DUP#0= Quit? OR
	    UNTIL
	;
	::  SetQuit			( *No room, so quit* )
	;

	#0<> Quit? AND
	ITE_DROP			( *Abort by Quit/ATTN* )
	    StIsDown			( *Abort by st. down* )

	Quit?				( *Until quit* )
    UNTIL
;

*** Byebye ******************************************************************
* Interface :	( --> {} )
* Description :	TETRIS game over sequence
* Modifies :	A Drop Quit Pv
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME	Byebye
::
    ClrQuit ClrPreV			( *Clear user flags* )

    ATTN? ITE
    :: 	ATTNFLGCLR FLUSHKEYS
	TURNMENUON RECLAIMDISP		( *Restore screen* )
    ;
    ::	FORTYFOUR TWENTYFIVE
	ASSEMBLE
	MKSGROB 40,7,0000000000C94470757324D6105515AD557055732554105215C5547072750000000000
	RPL
	XYGROBDISP			( *Show 'Game Over'* )

	5400 !A				( *Generate sound values* )
	THIRTEEN ZERO_DO (DO)
	    SIX @A DUP 400 #- !A
	LOOP
	THIRTEEN ZERO_DO (DO)		( *Generate sound* )
	    setbeep
	LOOP

	FLUSHKEYS WaitForKey 2DROP

	GetHSTab			( --> { { $n %s } .. } )

	TURNMENUON RECLAIMDISP		( *Restore screen* )

	SIX
	#1+_ONE_DO (DO)
	    DUP INDEX@ NTHCOMPDROP
	    TWO NTHCOMPDROP
	    @Score #>?SKIP
	    ::				( *$Prompt* )
		"One of the best \1F\nWelcome in the hall\nof fame. Who are you ?"
		NULL$			( *$EditLine* )
		{ ONE ONE }		( *CursorPos* )
		#TWO#ONE		( *#Ins/rep, #Entry* )
		ONE			( *Alphalock* )
		NULL{}			( *ILMenu* )
		ZEROFALSE		( *#ILMenuRow, AttnAbort?* )
		ZERO			( *#Parse* )
		InputLine
		NOTcase ExitAtLOOP

		@Score TWO{}N		( --> {hs} {} )
		OVER ONE INDEX@ #1- SUBCOMP SWAP >TCOMP
		SWAP INDEX@ FIVE SUBCOMP &COMP
		StoHS
		ExitAtLOOP
	    ;
	LOOP

	DrawHSTab
    ;

    ABND				( *Abandon game environment* )
;

*** UpdatVal ****************************************************************
* Interface :	( #y #val --> )
* Description :	Show val at 2,y in medium font on screen
* Ed. History :
*     1.000	18.08.1991	DM	start
*****************************************************************************

NULLNAME UpdatVal
::
    #>$ $>GROB TWO ROT			( --> grob #2 #y )
    HARDBUFF UNROT			( --> grob hb #x #y )
    GROB!
;

*** LinesUp *****************************************************************
* Interface :	( #pos --> #pos )
* Description :	Update game field, increment lines, correct level
* Modifies :	Lines
* Ed. History :
*     1.000	18.08.1991	DM	start
*	001	03.12.1991	DM	clear first row
*****************************************************************************

NULLNAME LinesUp
::
    DUP REALOBOB #/ SWAPDROP #2* #2* 	( #y*4 )
    HARDBUFF FORTYFOUR ZERO 84 5ROLL	( hb #44 #0 #84 #y )
    SUBGROB				( grob )
    HARDBUFF FORTYFOUR FOUR GROB!	( *Clear full line* )

    HARDBUFF FORTYFOUR ZERO 84 FOUR GROB!ZERODRP

    @Lines #1+DUP !Lines		( *Update full lines value* )
    FORTYTHREE OVER UpdatVal		( *Update screen* )

    FIFTEEN 400 TWENTYFOUR 300
    setbeep setbeep			( *Generate full line sound* )

    FIVE #/ SWAPDROP @Level #> NOT?SEMI	( *1 level up* )

    LevelUp
;

NULLNAME	LevelUp			( --> )
::
    TWENTYSIX @Level #1+DUP !Level	( *Update level value* )
    UpdatVal				( *Update screen* )
    @Timer % 0.0171 %- % 0.0001 %MAX
    !Timer				( *Update timer value* )
;

*** StIsDown ****************************************************************
* Interface :	( --> )
* Description :	Stone is down, so update score, check if lines are full, et c.
* Modifies :	Score
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME StIsDown
::
    @Score DUP
    @Level PreV? ITE TWO THREE #* TWENTYSEVEN #+ @StFt #- #+DUP
    !Score				( #o #n *Update score value* )

    NINE OVER UpdatVal

    1000 #/ SWAPDROPSWAP 1000 #/ SWAPDROP
    #<> IT				( *Next 1000* )
    ::	SEVEN 700 EIGHT 500
	setbeep setbeep			( *Generate sound* )
    ;

    #E0B				( *[11,14]* )
    BEGIN
	HXS F 000100200300400 OVER	( #[pos] hxs[fl] #[pos] )
	2DUP TstSt UNROT		( #[pos] #[f] hxs[fl] #[pos] )
	#5+				( *x += 5* )
	TstSt #+ FOUR #=ITE
	    LinesUp			( *Line full* )
	::  REALOBOB #-			( *--y* )
	;
	#200 OVER#>			( *2 > y* )
    UNTIL DROP

    FLUSHKEYS GARBAGE			( *Clr. key buf., force garbage coll.* )
;

*** ChkKeys *****************************************************************
* Interface :	( hxs(stim) #(p) --> hxs(stim) #(p) )
* Description :	Check for pending keystroce, perform action, perf. speed ctrl.
* Modifies :	Drop Pv Quit
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME	Actions
{
    ::				( *Rotate stone* )
				( hxs[sblk] #p --> hxs[sblk]' #p )
	ZERO SetSt			( *Clear stone* )

	Stones @StNo NTHCOMPDROP
	@StDir #1-
	DUP#0=IT			( *Get stone images, new direction* )
	::  DROP DUPLENCOMP
	;

	DUP !A				( *Store new direction* )
	NTHCOMPDROP SWAP		( hxs[i] hxs[i'] #[p] )

	2DUP TstSt #0=ITE
	::  ROTDROP @A !StDir
	;
	    SWAPDROP

	#F99F SetSt			( *Draw stone* )
    ;

    ::				( *Stone left* )
	ZERO SetSt
	MINUSONE MoveStX DROP
	#F99F SetSt
    ;

    ::				( *Stone right* )
	ZERO SetSt
	ONE MoveStX DROP
	#F99F SetSt
    ;

    ::				( *Stone left as possible* )
	ZERO SetSt
	BEGIN
	    MINUSONE MoveStX
	UNTIL
	#F99F SetSt
    ;

    ::				( *Stone right as possible* )
	ZERO SetSt
	BEGIN
	    ONE MoveStX
	UNTIL
	#F99F SetSt
    ;

    ::				( *Drop stone* )
	SetDrpReq
    ;

    Pause

    ::				( *Toggle stone preview* )
	THREE DUP TestUserFlag
	case
	::  ClrUserFlag			( *Preview off* )
	    @NxtStIm #C06 ZERO		( *Clr preview field* )
	    SetSt 2DROP
	;
	SetUserFlag
	@NxtStIm #C06 #F99F		( *Preview stone at [20,10]* )
	SetSt 2DROP
    ;

    LevelUp

    ::				( *Quit game* )
	SetQuit
    ;

    ::				( *Toggle sound* )
	FIFTYSIX DUP
	TestSysFlag
	case
	    ClrSysFlag
	SetSysFlag
    ;
}

NULLNAME	ChkKeys
::
    @Timer dowait			( *Perform speed control* )

    GETTOUCH NOT?SEMI

    H/W>KeyCode
    Actions
    {   kcUpArrow kcLeftArrow kcRightArrow
	kcVarsMenu kcNextRow kcDownArrow
	kcMenuKey6 kcMenuKey2 kcMenuKey5 kcMenuKey1
	kcNegate

*	kc8 kc4 kc6			( ** )
*	kc7 kc9 kc5			( ** )
    }
    ROT ' #= POSCOMP
*    ELEVEN OVER#< IT			( ** )
*    ::  ELEVEN #-			( ** )
*    ;					( ** )
    NTHELCOMP IT EVAL
;

*** MoveStX *****************************************************************
* Interface :	( hxs(sblk) #(p) #(stp) --> hxs(sblk) #(p') flag )
* Description :	Move stone x stp, stone left clear on screen
* Modifies :	A
* Ed. History :
*     1.000	08.08.1991	DM	start
*****************************************************************************

NULLNAME	MoveStX
::
    DUP !A				( *Store step* )
    #+					( *New stone pos* )
    2DUP TstSt #0=			( *Check if stone has room* )
    caseFALSE

    @A #- TRUE				( *No room, so restore pos* )
;

*** Pause *******************************************************************
* Interface :	( --> )
* Description :	Store, clear game area, show pause grob, wait for key,
*		restore game area.
* Ed. History :
*     1.000	09.09.1991	DM	start
*****************************************************************************

NULLNAME	Pause
::
    FORTYFOUR ZERO 2DUP HARDBUFF UNROT	( x1 y1 hb x1 y1 )
    84 SIXTY SUBGROB UNROT		( grob x1 y1 )
    2DUP HARDBUFF UNROT 84 SIXTY GROB!ZERODRP

    FORTYFIVE TWENTY
    ASSEMBLE
	MKGROB 37,39
	NIBHEX	080051000008005100000E30510000080051010008984201000C8842C70008EB
	NIBHEX	420100088CF70300049505810004D40501000A84C701000AA405820009C50782
	NIBHEX	0084E585450082B205450089A205290084B68252100FAA83941009AAC6391009
	NIBHEX	AA45521009AAAAEF0001F76D28000114AA28000F14C7EF00011444101001F74C
	NIBHEX	FF300908CF0020880828002084001010208FFFFFFF30C429429460A4294294A0
	NIBHEX	FFFFFFFFF110001100011008020001100802000110080200011008020001FFFF
	NIBHEX	FFFFF1
    RPL
    SIXTY THREE
    ASSEMBLE
	MKSGROB 7,17,8080E38081C0804122141422C1414141C1
    RPL
    XYGROBDISP XYGROBDISP
    WaitForKey 2DROP
    ROT XYGROBDISP
    GARBAGE
;

*** StoneHandler ************************************************************
* Interface :	( --> {} )
* Description :
* Ed. History :
*     0.000	xx.xx.1991	RH	alpha ver
*     1.000	08.08.1991	DM	start conversion
*	001	17.08.1991	DM	chg to support bint's
*	002	25.08.1991	DM	chg to support HXS x nnn
*****************************************************************************

NULLNAME StoneHandler
{
    ::			( *Test if room* hxs[image] #[pos] --> # )
	TOTEMPOB
	ABUFF

	CODE
* CPU			A	B	C	D0	D1	Rx
	C=DAT1	A			->ABUFF
	D1=D1+	5
	D=D+1	A
	C=C+CON	A,10
	C=C+CON	A,10
	R4=C							R4=->ABUFF

	CD0EX				rsp
	R2=C							R2=rsp
	BCEX	A			retstk
	R3=C							R3=retstk
	C=DAT1	A			&#pos
	D0=C					&#pos
	R1=C							R1=&#pos
	D0=D0+	5				&pos
	A=0	W
	A=DAT0	A	pos
	R0=A							R0=pos
	D=D+1	A	    *** Pop #[pos] from stack		    ***
	D1=D1+	5
	A=DAT1	A	&hxs[i]
	DAT1=C	A	    *** hxs becomes #			    ***
	AD0EX					&hxs[i]
	D0=D0+	5				&len
	C=0	W			0
	C=DAT0	2			len
	D0=D0+	5				&imag
	C=C-CON	A,6
	P=C	0
	C=DAT0	WP			imag
	A=C	W	imag
	B=0	W		0
TLoop	B=0	A
	P=	2
	B=C	WP		next coord.
	C=R0				pos
	C=C+B	A			offs
	B=C	A		offs
	BSR	A		>> 4
	P=	2
	B=C	P
	P=	0
	B=0	P
	BSRB.F	A
	P=	2
	C=0	P
	B=B+C	A
	P=	0
**	D0=(5)	=VDISP
**	C=DAT0	A
	C=R4.F	A
**	C=C+CON	A,10
**	C=C+CON	A,10
	C=C+B	A
	CD0EX
	C=0	W
	C=DAT0	P
	?C=0	P
	GOYES	TCont4
	B=B+1	S
	GOTO	TCont5
TCont4	P=	15
	LCHEX	8
	P=	0
	B=B!C	S
TCont5	ASR	W
	ASR	W
	ASR	W
	C=A	W
	?C#0	A
	GOYES	TLoop
	A=0	W
	C=0	W
	A=B	S
	ASLC
	?ABIT=1	3
	GOYES	TCont6
	C=C+1	A
TCont6	ABIT=0	3
	?A=0	B
	GOYES	TCont7
	C=C+1	A
TCont7	A=R1		&#pos
	AD0EX					&#pos
	D0=D0+	5				&pos
	DAT0=C	A
	C=R3				retstk
	BCEX	A		retstk
	C=R2				rsp
	CD0EX					rsp
	P=	0
	LOOP		    *** Jump to next RPL instr. in runstream  ***

	ENDCODE
    ;
    ::			( *Draw image* hxs[image] #[pos] #[patt] --> hxs pos )
	ABUFF

	CODE
*  CPU			A	B	C	D0	D1	R0	R1
	A=DAT1	A	->ABUFF
	D1=D1+	5
	D=D+1	A
	A=A+CON	A,10
	A=A+CON	A,10
	R2=A							R2=->ABUFF

	A=0	W	0
	GOSBVL	=POP#	patt
	GOSBVL	=SAVPTR
	R1=A								patt
	GOSBVL	=POP#	pos
	R0=A							pos
	A=DAT1	A	&hxs
	AD0EX					&hxs
	D0=D0+	10				&image
	C=0	W			0
	C=DAT0	12			image[12]
	A=C	W	image
XLoop	B=0	W		0
	P=	2
	B=C	WP		C[2]=x
	C=R0				pos
	C=C+B	A			pos+x
	B=C	W		pos+x
	BSR	W		>>= 4
	P=	2
	B=C	P
	P=	0
	B=0	P
	BSRB			B >>= 1
	P=	2
	C=0	P		C[2] = 0
	B=B+C	A		B[A] = offset in screen
	P=	0
**	D0=(5)	=VDISP
**	C=DAT0	A
	C=R2.F	A
**	C=C+CON	A,10
**	C=C+CON	A,10
	C=C+B	A
	D0=C
	C=R1			C = [patt]
	CSL	W		C <<= 4
	CSL	W		C <<= 4
	P=	4
XCont4	DAT0=C	XS
	CSR	W
	D0=D0+	16
	D0=D0+	16
	D0=D0+	2
	P=P-1
	?P#	0
	GOYES	XCont4
	ASR	W
	ASR	W
	ASR	W		A >>= 12
	C=A	W
	?C#0	A
	GOYES	XLoop
	P=	0
	GOSBVL	=GETPTRLOOP	Restore RPL values, dispatch to runstream

	ENDCODE
    ;
}

*** Stones ******************************************************************
* Interface :	( --> {} )
* Description :
* Ed. History :
*     0.000	xx.xx.1991	RH	alpha ver
*     1.000	08.08.1991	DM	start
*	001	25.08.1991	DM	chg to HXS C xx, correct pos
*****************************************************************************

NULLNAME Stones
{
    {   HXS C 301100300200
	HXS C 102202201200
	HXS C 101301201100
	HXS C 102200100101
    }
    {   HXS C 101300200100
	HXS C 202100200201
	HXS C 101301201300
	HXS C 202102101100
    }
    {   HXS C 201100300200
	HXS C 202101200201
	HXS C 101301200201
	HXS C 202301201200
    }
    {   HXS C 101300201200
	HXS C 302201301200
    }
    {   HXS C 301100201200
	HXS C 102201101200
    }
    {   HXS C 000300100200
	HXS C 103102101100
    }
    {   HXS C 201101100200
    }
}

*****************************************************************************
*		End of TETRIS.S
*****************************************************************************
